/*
 *  This file is part of xMule2.
 *  Copyright (C) 2003 Alo Sarv <madcat_@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef __GNUG__
	#pragma implementation "Images.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#include "Images.h"

CImages::CImages() {
	LoadImages();
}

/**
 * This method converts the paths we need to platform-dependant version.
 * Under wxMac, the separator is :, and relative paths must be prepended
 * by :. Also, for release builds, we add ::Resources, as it is standard
 * under wxMac to have resources there - packager must compile with 
 * -D__RELEASE__ and ensure the images are there.
 */
wxString CImages::MakePath(wxString path) {
	#if defined(__WXMAC__) && !defined(__MACH__)
		path.Replace(wxT("/"), wxT(":"));
		path.Prepend(wxT(":"));
		path.Prepend(wxT("::Resources"));
	#endif
	return path;
}

/**
 * This method loads all images into the global variables.
 */
void CImages::LoadImages() {
	frienduser.LoadFile(MakePath(wxT("images/default/friend.png")), wxBITMAP_TYPE_PNG);
	messages.LoadFile(MakePath(wxT("images/default/messages.png")), wxBITMAP_TYPE_PNG);
	search.LoadFile(MakePath(wxT("images/default/searchresults.png")), wxBITMAP_TYPE_PNG);
	provider.LoadFile(MakePath(wxT("images/default/provider.png")), wxBITMAP_TYPE_PNG);
	servers.LoadFile(MakePath(wxT("images/default/serverlist.png")), wxBITMAP_TYPE_PNG);
	newserver.LoadFile(MakePath(wxT("images/default/newserver.png")), wxBITMAP_TYPE_PNG);
	updateserver.LoadFile(MakePath(wxT("images/default/updateservers.png")), wxBITMAP_TYPE_PNG);
	info.LoadFile(MakePath(wxT("images/default/info.png")), wxBITMAP_TYPE_PNG);
	serverinfo.LoadFile(MakePath(wxT("images/default/serverinfo.png")), wxBITMAP_TYPE_PNG);
	log.LoadFile(MakePath(wxT("images/default/log.png")), wxBITMAP_TYPE_PNG);
	files.LoadFile(MakePath(wxT("images/default/sharedfiles.png")), wxBITMAP_TYPE_PNG);
	updown11.LoadFile(MakePath(wxT("images/default/updown11.png")), wxBITMAP_TYPE_PNG);
	updown00.LoadFile(MakePath(wxT("images/default/updown00.png")), wxBITMAP_TYPE_PNG);
	connected.LoadFile(MakePath(wxT("images/default/connectedhigh.png")), wxBITMAP_TYPE_PNG);
	disconnected.LoadFile(MakePath(wxT("images/default/disconnected.png")), wxBITMAP_TYPE_PNG);
	timer.LoadFile(MakePath(wxT("images/default/clock.png")), wxBITMAP_TYPE_PNG);
	user.LoadFile(MakePath(wxT("images/default/user.png")), wxBITMAP_TYPE_PNG);
	server.LoadFile(MakePath(wxT("images/default/server.png")), wxBITMAP_TYPE_PNG);
	small.LoadFile(MakePath(wxT("images/default/small.png")), wxBITMAP_TYPE_PNG);
	stats.LoadFile(MakePath(wxT("images/default/statistics.png")), wxBITMAP_TYPE_PNG);
	upload.LoadFile(MakePath(wxT("images/default/upload.png")), wxBITMAP_TYPE_PNG);
	download.LoadFile(MakePath(wxT("images/default/download.png")), wxBITMAP_TYPE_PNG);
	part.LoadFile(MakePath(wxT("images/default/partstats.png")), wxBITMAP_TYPE_PNG);
	full.LoadFile(MakePath(wxT("images/default/fullstats.png")), wxBITMAP_TYPE_PNG);
	exclamation.LoadFile(MakePath(wxT("images/default/exclamation.png")), wxBITMAP_TYPE_PNG);
	connection.LoadFile(MakePath(wxT("images/default/connection.png")), wxBITMAP_TYPE_PNG);
	daily.LoadFile(MakePath(wxT("images/default/daily.png")), wxBITMAP_TYPE_PNG);
	monthly.LoadFile(MakePath(wxT("images/default/monthly.png")), wxBITMAP_TYPE_PNG);
	yearly.LoadFile(MakePath(wxT("images/default/yearly.png")), wxBITMAP_TYPE_PNG);
	mule.LoadFile(MakePath(wxT("images/default/mule.png")), wxBITMAP_TYPE_PNG);
	btn_connect.LoadFile(MakePath(wxT("images/default/btn_connect.png")), wxBITMAP_TYPE_PNG);
	btn_servers.LoadFile(MakePath(wxT("images/default/btn_servers.png")), wxBITMAP_TYPE_PNG);
	btn_transfer.LoadFile(MakePath(wxT("images/default/btn_transfer.png")), wxBITMAP_TYPE_PNG);
	btn_search.LoadFile(MakePath(wxT("images/default/btn_search.png")), wxBITMAP_TYPE_PNG);
	btn_shared.LoadFile(MakePath(wxT("images/default/btn_shared.png")), wxBITMAP_TYPE_PNG);
	btn_messages.LoadFile(MakePath(wxT("images/default/btn_messages.png")), wxBITMAP_TYPE_PNG);
	btn_statistics.LoadFile(MakePath(wxT("images/default/btn_statistics.png")), wxBITMAP_TYPE_PNG);
	btn_guisettings.LoadFile(MakePath(wxT("images/default/btn_guisettings.png")), wxBITMAP_TYPE_PNG);
	btn_preferences.LoadFile(MakePath(wxT("images/default/btn_preferences.png")), wxBITMAP_TYPE_PNG);
	logo.LoadFile(MakePath(wxT("images/default/splashscreen.png")), wxBITMAP_TYPE_PNG);
	leftarrow.LoadFile(MakePath(wxT("images/default/leftarrow.png")), wxBITMAP_TYPE_PNG);
	rightarrow.LoadFile(MakePath(wxT("images/default/rightarrow.png")), wxBITMAP_TYPE_PNG);
}
