/*
 *  This file is part of xMule2.
 *  Copyright (C) 2003 Alo Sarv <madcat_@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef __GNUG__
	#pragma implementation "ServerWnd.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h". 
#include "wx/wxprec.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#include "ServerWnd.h"
#include "ServerListCtrl.h"
class CServerListCtrl *serverlistctrl;

/*
 * CServerWnd class.
 */

BEGIN_EVENT_TABLE(CServerWnd,wxPanel)
	EVT_BUTTON(ID_BTN_TOGGLE_SIDEBAR, CServerWnd::ToggleSidebar)
	EVT_BUTTON(ID_BTN_SHOW_ADDSERVER, CServerWnd::ToggleAddServerPanel)
	EVT_BUTTON(ID_BTN_SHOW_UPDATELIST, CServerWnd::ToggleUpdateServerPanel)
	EVT_BUTTON(ID_BTN_SHOW_SERVERSETTINGS, CServerWnd::ToggleServerSettingsPanel)
	EVT_BUTTON(ID_BTN_SHOW_PORT_OPTIONS, CServerWnd::TogglePortSettingsPanel)
	EVT_BUTTON(ID_BTN_SHOW_LOGGINGOPTIONS, CServerWnd::ToggleLogOptionsPanel)
	EVT_BUTTON(ID_ADDTOLIST, CServerWnd::AddNewServer)
	EVT_BUTTON(ID_UPDATE, CServerWnd::UpdateFromURL)
	EVT_SIZE(CServerWnd::OnSize)
END_EVENT_TABLE()

/* 
 * Constructor for server window. Loads the server dialog and sets LogBook 
 * images.
 */
CServerWnd::CServerWnd( 
	wxWindow *parent, wxWindowID id, const wxPoint &position, 
	const wxSize& size, long style ) 
: wxPanel( parent, id, position, size, style ) {
	bool show_sidebar;

	CreateControls();
	
	/**
	 *  Read SideBar value from config object, and if it returns false, toggle (hide)
	 * the sidebar.
	 */
	m_config->Read(wxT("/ServerWnd/SideBar"), &show_sidebar, true);
	if (!show_sidebar) {
		wxCommandEvent null_event;
		ToggleSidebar(null_event);
	}

	Hide();

	/* Set the images of static bitmaps on this page. */
	GetServerListImage()->SetBitmap(img->servers);
	GetSSBitmapProvider()->SetBitmap(img->provider);
	GetBtnToggleSidebar()->SetBitmapLabel(img->leftarrow);

	/* Set log notebook page images */
	m_imagelist = new wxImageList(16, 16);
	m_imagelist->Add(img->serverinfo);
	m_imagelist->Add(img->log);
	
	GetLogBook()->AssignImageList(m_imagelist);            
	GetLogBook()->SetPageImage(0, 0);
	GetLogBook()->SetPageImage(1, 1);
	GetLogBook()->SetPageImage(2, 1);
}

/*
 * Destructor for server window. All shutdown functions go here.
 */
CServerWnd::~CServerWnd() {
	/**
	 * We need to check here if the values GetSashPosition() returns us are
	 * correct, because if user never switched to Server page during this
	 * session, they return some absurd values. In that case, simply don't
	 * save them.
	 */
	m_config->SetPath(wxT("/ServerWnd"));
	if (GetHorizontalSplitter()->GetSashPosition() > 10) {
		m_config->Write(wxT("Horizontal_Splitter"),
			GetHorizontalSplitter()->GetSashPosition());
	}
	
	m_config->Write(wxT("SideBar"), sidebar->IsShown());
	m_config->Write(wxT("SideBar_AddServer"), addserverpanel->IsShown());
	m_config->Write(wxT("SideBar_UpdateServer"),updateserverpanel->IsShown());
	m_config->Write(wxT("SideBar_ServerSettings"), settingspanel->IsShown());
	m_config->Write(wxT("SideBar_PortSettings"), portsettingspanel->IsShown());
	m_config->Write(wxT("SideBar_LoggingOptions"),logoptionspanel->IsShown());
}

/**
 * These three almost identical methods add a line of [text] into the
 * corresponding log listctrls, optionally also displaying it in statusbar.
 * All lines will be added current date/time before it, and listctrls
 * are scrolled to display the last added line.
 */
void CServerWnd::AddDebugLine(wxString text, bool status) {
	if (status) {
		statusbar->SetLogText(text);
	}
	
	wxDateTime now = wxDateTime::Now();
	text = wxString::Format(
		wxT("[%s] %s\n"), now.Format(wxT("%c")).c_str(), text.c_str()
	);
	
	GetDebugLog()->AppendText(text);
	GetDebugLog()->ShowPosition(GetDebugLog()->GetLastPosition());
}

void CServerWnd::AddLogLine(wxString text, bool status) {
	if (status) {
		statusbar->SetLogText(text);
	}
	
	wxDateTime now = wxDateTime::Now();
	text = wxString::Format(
		wxT("[%s] %s\n"), now.Format(wxT("%c")).c_str(), text.c_str()
	);
	
	GetLog()->AppendText(text);
	GetLog()->ShowPosition(GetLog()->GetLastPosition());
}

void CServerWnd::AddServerLogLine(wxString text, bool status) {
	if (status) {
		statusbar->SetLogText(text);
	}
	
	wxDateTime now = wxDateTime::Now();
	text = wxString::Format(
		wxT("[%s] %s\n"), now.Format(wxT("%c")).c_str(), text.c_str()
	);
	
	GetServerLog()->AppendText(text);
	GetServerLog()->ShowPosition(GetServerLog()->GetLastPosition());
}

/*
 * Add new server to servers list.
 */
void CServerWnd::AddNewServer() {
	AddDebugLine(wxT("Add New Server button clicked."));
}

/*
 * Downloads the server.met file from given URL and adds any new servers found
 * to server list.
 */
void CServerWnd::UpdateFromURL() {
	AddDebugLine(wxT("Update Server List from URL button clicked."));
}

/* Creates server page controls */
void CServerWnd::CreateControls() {
	int horizontal_splitter_pos;
	bool show_addserver, show_updateserver, show_serversettings, 
		show_portsettings, show_logoptions;
	wxString closed;

	/* Flexgridsizer with two columns, second of them being growable. */
	mainsizer = new wxFlexGridSizer(3);
	mainsizer->AddGrowableCol( 2 );
	mainsizer->AddGrowableRow( 0 );

	/* First add the sidebar, its going to be the left-most control. */
	sidebar = new wxPanel(
		this, -1, wxDefaultPosition, wxDefaultSize,
		wxNO_FULL_REPAINT_ON_RESIZE|wxRAISED_BORDER);
	Server_Sidebar(sidebar, true, true);
	mainsizer->Add(sidebar, 0, wxGROW|wxTOP|wxBOTTOM, 5);

	/* Create sidebar panels */
	addserverpanel = new wxPanel(sidebar, -1);
	updateserverpanel = new wxPanel(sidebar, -1);
	settingspanel = new wxPanel(sidebar, -1);
	portsettingspanel = new wxPanel(sidebar, -1);
	logoptionspanel = new wxPanel(sidebar, -1);

	/* Insert content into them */
	Server_AddServerPanel(addserverpanel, true, true);
	Server_UpdatePanel(updateserverpanel, true, true);
	Server_SettingsPanel(settingspanel, true, true);
	Server_PortSettingsPanel(portsettingspanel, true, true);
	Server_LogOptionsPanel(logoptionspanel, true, true);

	/* Set button labels */
	closed = wxT(" >>");
	GetBtnToggleAddServer()->SetLabel(_("Add a server") + closed);
	GetBtnToggleUpdateServer()->SetLabel(_("Update from URL") + closed);
	GetBtnToggleServerSettings()->SetLabel(_("Server settings") + closed);
	GetBtnTogglePortSettings()->SetLabel(_("Port settings") + closed);
	GetBtnToggleLogOptions()->SetLabel(_("Logging options") + closed);
	
	/* Hide all of them */
	addserverpanel->Hide();
	updateserverpanel->Hide();
	settingspanel->Hide();
	portsettingspanel->Hide();
	logoptionspanel->Hide();

	/* Read from config object which ones should be shown */
	m_config->Read(wxT("/ServerWnd/SideBar_AddServer"), &show_addserver, false);
	m_config->Read(wxT("/ServerWnd/SideBar_UpdateServer"), &show_updateserver, false);
	m_config->Read(wxT("/ServerWnd/SideBar_ServerSettings"), &show_serversettings, true);
	m_config->Read(wxT("/ServerWnd/SideBar_PortSettings"), &show_portsettings, false);
	m_config->Read(wxT("/ServerWnd/SideBar_LoggingOptions"), &show_logoptions, true);

	/* Call Toggle methods with null event as needed */
	wxCommandEvent null_event;
	if (show_addserver) {
		ToggleAddServerPanel(null_event);
	}
	if (show_updateserver) {
		ToggleUpdateServerPanel(null_event);
	}
	if (show_serversettings) {
		ToggleServerSettingsPanel(null_event);
	}
	if (show_portsettings) {
		TogglePortSettingsPanel(null_event);
	}
	if (show_logoptions) {
		ToggleLogOptionsPanel(null_event);
	}

	/* Bitmap button between sidebar and splitter window to toggle sidebar on/off */
	wxBitmapButton *toggler = new wxBitmapButton(
		this, ID_BTN_TOGGLE_SIDEBAR, img->leftarrow, 
		wxDefaultPosition, wxSize(10,100)
	);
	mainsizer->Add(toggler, 0, wxALIGN_CENTER, 5 );
	
	/* Read splitter positions from configuration object. */
	m_config->Read(wxT("/ServerWnd/Horizontal_Splitter"),
		&horizontal_splitter_pos, 400);

	/* Splitter window */
	splatter = new wxSplitterWindow(
		this, ID_SERVER_SPLITTER_HORIZONTAL, wxDefaultPosition, 
		wxDefaultSize, wxSP_3DSASH|wxSP_NOBORDER|wxSP_LIVE_UPDATE|
		wxNO_FULL_REPAINT_ON_RESIZE
	);
	splatter->SetMinimumPaneSize(10);

	/* Top Panel */
	top_panel = new wxPanel(splatter, -1,
		wxDefaultPosition, wxDefaultSize,
		wxNO_FULL_REPAINT_ON_RESIZE|wxTAB_TRAVERSAL
	);
	Server_List(top_panel, false, true);

	/* Bottom Panel */
	bottom_panel = new wxPanel(splatter, -1,
		wxDefaultPosition, wxDefaultSize,
		wxNO_FULL_REPAINT_ON_RESIZE|wxTAB_TRAVERSAL);
	Server_Logs( bottom_panel, false, true);

	/* Split the window and add to sizer */
	splatter->SplitHorizontally( top_panel, bottom_panel,
		horizontal_splitter_pos);
	mainsizer->Add(splatter, 0, wxGROW, 0 );

	this->SetAutoLayout( TRUE );
	this->SetSizer( mainsizer );
}

/**
 * These 4 methods toggle the displayment of sidebar panels by determining wether
 * its shown or not by call to IsShown(), and then either removing it from sizer
 * and hiding it, or attaching it to sizer and showing it. Layout() is needed to
 * recalculate item positions, and Refresh() to avoid strange GUI bugs.
 */
void CServerWnd::ToggleAddServerPanel(wxCommandEvent &event) {
	wxString addserver;
	
	addserver = _("Add a server");
	
	if (addserverpanel->IsShown()) {
		sizer_addserver->Remove(addserverpanel);
		addserverpanel->Hide();
		GetBtnToggleAddServer()->SetLabel(addserver+wxT(" >>"));
	} else {
		sizer_addserver->Add(addserverpanel, 0, wxGROW, 0);
		addserverpanel->Show();
		GetBtnToggleAddServer()->SetLabel(addserver+wxT(" <<"));
	}
	
	sidebar->Layout();
	sidebar->Refresh();
}

void CServerWnd::ToggleUpdateServerPanel(wxCommandEvent &event) {
	wxString updateurl;
	
	updateurl = _("Update from URL");
	
	if (updateserverpanel->IsShown()) {
		sizer_updateserver->Remove(updateserverpanel);
		updateserverpanel->Hide();
		GetBtnToggleUpdateServer()->SetLabel(updateurl+wxT(" >>"));
	} else {
		sizer_updateserver->Add(updateserverpanel, 0, wxGROW, 0);
		updateserverpanel->Show();
		GetBtnToggleUpdateServer()->SetLabel(updateurl+wxT(" <<"));
	}
	
	sidebar->Layout();
	sidebar->Refresh();
}

void CServerWnd::ToggleServerSettingsPanel(wxCommandEvent &event) {
	wxString serversettings;
	
	serversettings = _("Server settings");
	
	if (settingspanel->IsShown()) {
		sizer_serversettings->Remove(settingspanel);
		settingspanel->Hide();
		GetBtnToggleServerSettings()->SetLabel(serversettings+wxT(" >>"));
	} else {
		sizer_serversettings->Add(settingspanel, 0, wxGROW, 0);
		settingspanel->Show();
		GetBtnToggleServerSettings()->SetLabel(serversettings+wxT(" <<"));
	}
	
	sidebar->Layout();
	sidebar->Refresh();
}

void CServerWnd::TogglePortSettingsPanel(wxCommandEvent &event) {
	wxString portsettings;
	
	portsettings = _("Port settings");
	
	if (portsettingspanel->IsShown()) {
		sizer_portsettings->Remove(portsettingspanel);
		portsettingspanel->Hide();
		GetBtnTogglePortSettings()->SetLabel(portsettings+wxT(" >>"));
	} else {
		sizer_portsettings->Add(portsettingspanel, 0, wxGROW, 0);
		portsettingspanel->Show();
		GetBtnTogglePortSettings()->SetLabel(portsettings+wxT(" <<"));
	}
	
	sidebar->Layout();
	sidebar->Refresh();
}

void CServerWnd::ToggleLogOptionsPanel(wxCommandEvent &event) {
	wxString loggingoptions;
	
	loggingoptions = _("Logging options");
	
	if (logoptionspanel->IsShown()) {
		sizer_logoptions->Remove(logoptionspanel);
		logoptionspanel->Hide();
		GetBtnToggleLogOptions()->SetLabel(loggingoptions+wxT(" >>"));
	} else {
		sizer_logoptions->Add(logoptionspanel, 0, wxGROW, 0);
		logoptionspanel->Show();
		GetBtnToggleLogOptions()->SetLabel(loggingoptions+wxT(" <<"));
	}
	
	sidebar->Layout();
	sidebar->Refresh();
}

/**
 * Toggles the displayment of sidebar. 
 * This code part requires a patched version of wxWindows that has 
 * wxFlexGridSizer::RemoveGrowableCol() method implemented (2.4.1 doesn't 
 * have it, and CVS doesn't have it either as of 2003/09/04).
 */ 
void CServerWnd::ToggleSidebar(wxCommandEvent &event) {
#ifdef __HAVE_REMOVE_GROWABLE_COL__
	if (sidebar->IsShown()) {
		mainsizer->Remove(sidebar);
		mainsizer->RemoveGrowableCol(2);
		mainsizer->AddGrowableCol(1);
		sidebar->Hide();
		GetBtnToggleSidebar()->SetBitmapLabel(img->rightarrow);
		GetBtnToggleSidebar()->SetBitmapSelected(img->rightarrow);
		GetBtnToggleSidebar()->SetBitmapFocus(img->rightarrow);
	} else {
		mainsizer->Prepend(sidebar, 0, wxGROW|wxTOP|wxBOTTOM, 5);
		mainsizer->RemoveGrowableCol(1);
		mainsizer->AddGrowableCol(2);
		sidebar->Show();
		GetBtnToggleSidebar()->SetBitmapLabel(img->leftarrow);
		GetBtnToggleSidebar()->SetBitmapSelected(img->leftarrow);
		GetBtnToggleSidebar()->SetBitmapFocus(img->leftarrow);
	}
	Layout();
	Refresh();
#endif
}
