/*
 *  This file is part of xMule2.
 *  Copyright (C) 2003 Alo Sarv <madcat_@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef __GNUG__
	#pragma implementation "StatisticsTreeCtrl.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#include "StatisticsTreeCtrl.h"

BEGIN_EVENT_TABLE(CStatisticsTreeCtrl,wxTreeCtrl)
END_EVENT_TABLE()

/*
 * Constructor for Statistics Tree Control. Sets up the root of the tree, 
 * assigns imagelist and calls sub-methods for filling in the rest of the
 * tree.
 */
CStatisticsTreeCtrl::CStatisticsTreeCtrl( wxWindow *parent, wxWindowID id,
    const wxPoint &position, const wxSize& size, long style 
) : wxTreeCtrl( parent, id, position, size, style ) {

	wxImageList *tree_images = new wxImageList(16, 16);
	tree_images->Add(img->updown11);	// 0  Up/Down
	tree_images->Add(img->connected);	// 1  Connection
	tree_images->Add(img->timer);		// 2 Clock
	tree_images->Add(img->user);		// 3  User
	tree_images->Add(img->server);		// 4  Server
	tree_images->Add(img->files);		// 5  Shared Files
	tree_images->Add(img->small);		// 6  Small Item
	tree_images->Add(img->stats);		// 7  Statistics
	tree_images->Add(img->upload);		// 8  Uploads
	tree_images->Add(img->download);	// 9  Downloads
	tree_images->Add(img->part);		// 10 Statistics Session
	tree_images->Add(img->full);		// 11 Statistics Total
	tree_images->Add(img->exclamation);	// 12 Records
	tree_images->Add(img->connection);	// 13 Prefs.Connection
	tree_images->Add(img->daily);		// 14 Daily
	tree_images->Add(img->monthly);		// 15 Monthly
	tree_images->Add(img->yearly);		// 16 Yearly
	AssignImageList(tree_images);
	
	wxTreeItemId root = AddRoot(wxT("xMule2 Statistics"), 7);
	SetItemBold(root);
	CreateTransferBranch(root);
	CreateConnectionBranch(root);
	CreateTimeStatisticsBranch(root);
	CreateClientsBranch(root);
	CreateServersBranch(root);
	CreateSharedFilesBranch(root);
}

/*
 * Fills the Transfer branch of the statistics tree.
 */
void CStatisticsTreeCtrl::CreateTransferBranch(wxTreeItemId root) {
	wxTreeItemId t_transfer = AppendItem(root, wxT("Transfer"), 0);
	SetItemBold(t_transfer);
		/* Transfer branch */
		AppendItem(t_transfer, wxT("Session UL:DL Ratio: 1:1"), 6);
		AppendItem(t_transfer, wxT("Comulative UL:DL Ratio: 1:1"), 6);
		wxTreeItemId t_uploads = AppendItem(t_transfer, wxT("Uploads"), 8);
		SetItemBold(t_uploads);
			wxTreeItemId t_session = AppendItem(t_uploads, wxT("Session"), 10);
			SetItemBold(t_session);
				/* Transfer->Uploads->Session branch */
				AppendItem(t_session, wxT("Uploaded Data: 0.0 MB"), 6);
				AppendItem(t_session, wxT("Active Uploads: 0"), 6);
				AppendItem(t_session, wxT("Waiting Uploads: 0"), 6);
				AppendItem(t_session, wxT("Upload Sessions: 0"), 6);
				AppendItem(t_session, wxT("Total Overhead (Packets): 0.0 MB (0.0 MB)"), 6);
			wxTreeItemId t_comulative = AppendItem(t_uploads, wxT("Comulative"), 11);
			SetItemBold(t_comulative);
				/* Transfer->Uploads->Comulative branch */
				AppendItem(t_comulative, wxT("Uploaded Data: 0.0 MB"), 6);
				AppendItem(t_comulative, wxT("Upload Sessions"), 6);
				AppendItem(t_comulative, wxT("Total Overhead (Packets): 0.0 MB, (0.0 MB)"), 6);
		wxTreeItemId t_downloads = AppendItem(t_transfer, wxT("Downloads"), 9);
		SetItemBold(t_downloads);
			/* Transfer->Downloads branch */
			AppendItem(t_downloads, wxT("Session"), 10);
			AppendItem(t_downloads, wxT("Comulative"), 11);
}

/*
 * Fills the Connections branch of the statistics tree.
 */
void CStatisticsTreeCtrl::CreateConnectionBranch(wxTreeItemId root) {
	wxTreeItemId t_connection = AppendItem(root, wxT("Connection"), 1);
	SetItemBold(t_connection);
		wxTreeItemId t_sess = AppendItem(t_connection, wxT("Session"), 10);
		SetItemBold(t_sess);
			wxTreeItemId t_sess_gen = AppendItem(t_sess, wxT("General"), 13);
			SetItemBold(t_sess_gen);
				AppendItem(t_sess_gen, wxT("Reconnects: 0"), 6);
				AppendItem(t_sess_gen, wxT("Active Connections (estimate): 0"), 6);
				AppendItem(t_sess_gen, wxT("Average Connections (estimate): 0"), 6);
				AppendItem(t_sess_gen, wxT("Peak Connections (estimate): 0"), 6);
				AppendItem(t_sess_gen, wxT("Max Connection Limit Reached: 0"), 6);
			wxTreeItemId t_sess_up = AppendItem(t_sess, wxT("Upload"), 8);
			SetItemBold(t_sess_up);
				AppendItem(t_sess_up, wxT("Upload-Speed: 0.0 kb/s"), 6);
				AppendItem(t_sess_up, wxT("Average Uploadrate: 0.0 kb/s"), 6);
				AppendItem(t_sess_up, wxT("Max Upload Rate: 0.0 kb/s"), 6);
				AppendItem(t_sess_up, wxT("Max Average Upload Rate: 0.0 kb/s"), 6);
			wxTreeItemId t_sess_down = AppendItem(t_sess, wxT("Download"), 9);
			SetItemBold(t_sess_down);
				AppendItem(t_sess_down, wxT("Download-Speed: 0.0 kb/s"), 6);
				AppendItem(t_sess_down, wxT("Average Downloadrate: 0.0 kb/s"), 6);
				AppendItem(t_sess_down, wxT("Max Download Rate: 0.0 kb/s"), 6);
				AppendItem(t_sess_down, wxT("Max Average Download Rate: 0.0 kb/s"), 6);
		wxTreeItemId t_com = AppendItem(t_connection, wxT("Comulative"), 11);
		SetItemBold(t_com);
			wxTreeItemId t_com_gen = AppendItem(t_com, wxT("General"), 13);
			SetItemBold(t_com_gen);
				AppendItem(t_com_gen, wxT("Reconnects: 0"), 6);
				AppendItem(t_com_gen, wxT("Active Connections (estimate): 0"), 6);
				AppendItem(t_com_gen, wxT("Average Connections (estimate): 0"), 6);
				AppendItem(t_com_gen, wxT("Peak Connections (estimate): 0"), 6);
				AppendItem(t_com_gen, wxT("Max Connection Limit Reached: 0"), 6);
			wxTreeItemId t_com_up = AppendItem(t_com, wxT("Upload"), 8);
			SetItemBold(t_com_up);
				AppendItem(t_com_up, wxT("Upload-Speed: 0.0 kb/s"), 6);
				AppendItem(t_com_up, wxT("Average Uploadrate: 0.0 kb/s"), 6);
				AppendItem(t_com_up, wxT("Max Upload Rate: 0.0 kb/s"), 6);
				AppendItem(t_com_up, wxT("Max Average Upload Rate: 0.0 kb/s"), 6);
			wxTreeItemId t_com_down = AppendItem(t_com, wxT("Download"), 9);
			SetItemBold(t_com_down);
				AppendItem(t_com_down, wxT("Download-Speed: 0.0 kb/s"), 6);
				AppendItem(t_sess_down, wxT("Average Downloadrate: 0.0 kb/s"), 6);
				AppendItem(t_com_down, wxT("Max Download Rate: 0.0 kb/s"), 6);
				AppendItem(t_com_down, wxT("Max Average Download Rate: 0.0 kb/s"), 6);
}

/*
 * Fills the Time Statistics branch of the statistics tree.
 */
void CStatisticsTreeCtrl::CreateTimeStatisticsBranch(wxTreeItemId root) {
	wxTreeItemId t_statistics = AppendItem(root, wxT("Time Statistics"), 2);
	SetItemBold(t_statistics);
		AppendItem(t_statistics, wxT("Statistics last reset: Unknown"), 6);
		AppendItem(t_statistics, wxT("Time since last reset: Unknown"), 6);
		wxTreeItemId t_sess = AppendItem(t_statistics, wxT("Session"), 10);
		SetItemBold(t_sess);
			AppendItem(t_sess, wxT("Runtime: 0 minutes"), 6);
			wxTreeItemId t_sess_ttime = AppendItem(t_sess, wxT("Transfer Time: 0 minutes (0.0%)"), 6);
				AppendItem(t_sess_ttime, wxT("Upload Time: 0:00 hours (0.0%)"), 6);
				AppendItem(t_sess_ttime, wxT("Download Time: 0:00 hours (0.0%)"), 6);
			AppendItem(t_sess, wxT("Current Server Duration: 0:00 hours (0.0%)"), 6);
			AppendItem(t_sess, wxT("Total Server Duration: 0:00 hours (0.0%)"), 6);
		wxTreeItemId t_com = AppendItem(t_statistics, wxT("Comulative"), 11);
		SetItemBold(t_com);
			AppendItem(t_com, wxT("Runtime: 0 minutes"), 6);
			wxTreeItemId t_com_ttime = AppendItem(t_com, wxT("Transfer Time: 0 minutes (0.0%)"), 6);
				AppendItem(t_com_ttime, wxT("Upload Time: 0:00 hours (0.0%)"), 6);
				AppendItem(t_com_ttime, wxT("Download Time: 0:00 hours (0.0%)"), 6);
			AppendItem(t_com, wxT("Total Server Duration: 0:00 hours (0.0%)"), 6);
		wxTreeItemId t_avg = AppendItem(t_statistics, wxT("Projected Averages"), 7);
		SetItemBold(t_avg);
			AppendItem(t_avg, wxT("Daily"), 14);
			AppendItem(t_avg, wxT("Monthly"), 15);
			AppendItem(t_avg, wxT("Yearly"), 16);
}

/*
 * Fills the Clients branch of the statistics tree.
 */
void CStatisticsTreeCtrl::CreateClientsBranch(wxTreeItemId root) {
	wxTreeItemId t_clients = AppendItem(root, wxT("Clients"), 3);
	SetItemBold(t_clients);
		wxTreeItemId t_soft = AppendItem(t_clients, wxT("Client Software"), 6);
			wxTreeItemId t_emule = AppendItem(t_soft, wxT("eMule: 0 (0.0%)"), 6);
				AppendItem(t_emule, wxT("v0.30: 0 (0.0%)"), 6);
				AppendItem(t_emule, wxT("v0.29: 0 (0.0%)"), 6);
				AppendItem(t_emule, wxT("v0.28: 0 (0.0%)"), 6);
				AppendItem(t_emule, wxT("v0.27: 0 (0.0%)"), 6);
				wxTreeItemId t_mule_old = AppendItem(t_emule, wxT("Old 0 (0.0%)"), 6);
					AppendItem(t_mule_old, wxT("v0.26: 0 (0.0%)"), 6);
					AppendItem(t_mule_old, wxT("v0.25: 0 (0.0%)"), 6);
					AppendItem(t_mule_old, wxT("v0.24: 0 (0.0%)"), 6);
					AppendItem(t_mule_old, wxT("v0.23: 0 (0.0%)"), 6);
			wxTreeItemId t_hybrid = AppendItem(t_soft, wxT("eD Hybrid: 0 (0.0%)"), 6);
				AppendItem(t_hybrid, wxT("v50.1: 0 (0.0%)"), 6);
				AppendItem(t_hybrid, wxT("v50.0: 0 (0.0%)"), 6);
				AppendItem(t_hybrid, wxT("v49.9: 0 (0.0%)"), 6);
				AppendItem(t_hybrid, wxT("v49.5: 0 (0.0%)"), 6);
				wxTreeItemId t_hybrid_old = AppendItem(t_hybrid, wxT("Old: 0 (0.0%)"), 6);
					AppendItem(t_hybrid_old, wxT("v49.4: 0 (0.0)"), 6);
					AppendItem(t_hybrid_old, wxT("v48.1: 0 (0.0)"), 6);
					AppendItem(t_hybrid_old, wxT("v48.0: 0 (0.0)"), 6);
					AppendItem(t_hybrid_old, wxT("v47.0: 0 (0.0)"), 6);
			wxTreeItemId t_donkey = AppendItem(t_soft, wxT("eDonkey: 0 (0.0%)"), 6);
				AppendItem(t_donkey, wxT("v61.0: 0 (0.0%)"), 6);
				AppendItem(t_donkey, wxT("v60.0: 0 (0.0%)"), 6);
				AppendItem(t_donkey, wxT("v59.0: 0 (0.0%)"), 6);
			wxTreeItemId t_xmule = AppendItem(t_soft, wxT("xMule: 0 (0.0%)"), 6);
				AppendItem(t_xmule, wxT("xMule 1.x: 0 (0.0%)"), 6);
				AppendItem(t_xmule, wxT("xMule 2.x: 0 (0.0%)"), 6);
			AppendItem(t_soft, wxT("MLdonkey: 0 (0.0%)"), 6);
			AppendItem(t_soft, wxT("cDonkey: 0 (0.0%)"), 6);
			AppendItem(t_soft, wxT("Shareaza: 0 (0.0%)"), 6);
			AppendItem(t_soft, wxT("Unknown: 0"), 6);
		wxTreeItemId t_port = AppendItem(t_clients, wxT("Port"), 6);
			AppendItem(t_port, wxT("Default: 0 (0.0%)"), 6);
			AppendItem(t_port, wxT("Other: 0 (0.0%)"), 6);
		AppendItem(t_clients, wxT("Secure Ident (OK:Failed): 0(0.0%):0(0.0%)"), 6);
		AppendItem(t_clients, wxT("Problematic: 0"), 6);
		AppendItem(t_clients, wxT("Banned: 0"), 6);
		AppendItem(t_clients, wxT("Filtered: 0"), 6);
}

/*
 * Fills the Servers branch of the statistics tree.
 */
void CStatisticsTreeCtrl::CreateServersBranch(wxTreeItemId root) {
	wxTreeItemId t_servers = AppendItem(root, wxT("Servers"), 4);
	SetItemBold(t_servers);
		wxTreeItemId t_workingservers = AppendItem(t_servers, wxT("Working Servers: 0"), 6);
			AppendItem(t_workingservers, wxT("Users on Working Servers: 0"), 6);
			AppendItem(t_workingservers, wxT("Files on Working Servers: 0"), 6);
			AppendItem(t_workingservers, wxT("Server Occupation: 0.0%"), 6);
		AppendItem(t_servers, wxT("Failed Servers: 0"), 6);
		AppendItem(t_servers, wxT("Deleted Servers: 0"), 6);
		AppendItem(t_servers, wxT("Total: 0"), 6);
		AppendItem(t_servers, wxT("Total Users: 0"), 6);
		AppendItem(t_servers, wxT("Total Files: 0"), 6);
		wxTreeItemId t_records = AppendItem(t_servers, wxT("Records"), 12);
		SetItemBold(t_records);
			AppendItem(t_records, wxT("Max. Working Servers: 0"), 6);
			AppendItem(t_records, wxT("Max. Users Online: 0"), 6);
			AppendItem(t_records, wxT("Max. Files Available: 0"), 6);
}

/*
 * Fills the Shared Files branch of the tree.
 */
void CStatisticsTreeCtrl::CreateSharedFilesBranch(wxTreeItemId root) {
	wxTreeItemId t_shared = AppendItem(root, wxT("Shared Files"), 5);
	SetItemBold(t_shared);
		AppendItem(t_shared, wxT("Number of Shared Files: 0"), 6);
		AppendItem(t_shared, wxT("Average filesize: 0.0 MB"), 6);
		AppendItem(t_shared, wxT("Largest Shared File: 0.0 MB"), 6);
		AppendItem(t_shared, wxT("Total size of Shared Files: 0.0 MB"), 6);
		wxTreeItemId t_records = AppendItem(t_shared, wxT("Records"), 12);
		SetItemBold(t_records);
			AppendItem(t_records, wxT("Max. Files Ever Shared: 0"), 6);
			AppendItem(t_records, wxT("Largest Average File Size: 0.0 MB"), 6);
			AppendItem(t_records, wxT("Largest Shared File: 0.0 MB"), 6);
			AppendItem(t_records, wxT("Largest Share Size: 0.0 MB"), 6);
}

