/*
 *  This file is part of xMule2.
 *  Copyright (C) 2003 Alo Sarv <madcat_@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef __GNUG__
	#pragma implementation "StatusBar.h"
#endif

// For compilers that support precompilation
#include "wx/wxprec.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#include "StatusBar.h"

/*
 * StatusBar class is a custom class for displaying the status bar on the bottom
 * of the dialog window. It loads the images, sets the texts and moves them 
 * around as window size changes.
 */
 
BEGIN_EVENT_TABLE(CStatusBar, wxStatusBar)
	EVT_SIZE(CStatusBar::OnSize)
END_EVENT_TABLE()

/*
 * Constructor for our custom status bar. Sets up four fields and creates the
 * bitmaps/textcontrols with default values.
 */
CStatusBar::CStatusBar(wxWindow *parent) : wxStatusBar(parent, -1) {

	SetFieldsCount(6);
	
	m_logtxt = new wxStaticText(
		this, -1, _("xMule v2 ready!"), wxDefaultPosition, 
		wxDefaultSize, wxST_NO_AUTORESIZE
	);

	m_userbmp = new wxStaticBitmap(this, -1, img->user);
	m_usertxt = new wxStaticText(this, -1, wxT(""));
	
	m_filesbmp = new wxStaticBitmap(this, -1, img->files);
	m_filestxt = new wxStaticText(this, -1, wxT(""));
	
	m_upbmp = new wxStaticBitmap(this, -1, img->upload);
	m_uptxt = new wxStaticText(this, -1, wxT(""));
	
	m_downbmp = new wxStaticBitmap(this, -1, img->download);
	m_downtxt = new wxStaticText(this, -1, wxT(""));
	
	m_serverbmp = new wxStaticBitmap(this, -1, img->disconnected);
	m_servertxt = new wxStaticText(this, -1, wxT(""));

	SetUserCount(0, 0);
	SetFilesCount(0, 0);
	SetUpSpeed(0);
	SetDownSpeed(0);
	SetServerName(wxT("Disconnected"));
}

/*
 * Destructor for our custom statusbar.
 */
CStatusBar::~CStatusBar() {
}

/*
 * OnSize event function. Here we recalculate the positions of all items and
 * place them into right locations relative to statusbar rectangles.
 */
void CStatusBar::OnSize(wxSizeEvent& event) {
	wxRect rect;
	
	GetFieldRect(0, rect);
	m_logtxt->SetSize(rect.x+2, rect.y+2, rect.width-4, rect.height-4);
	m_logtxt->Move(rect.x + 2, rect.y + 2);

	GetFieldRect(1, rect);
	m_usertxt->SetSize(rect.x+2, rect.y+2, rect.width-20, rect.height-4);
	m_userbmp->Move(rect.x + 2, rect.y + 2);
	m_usertxt->Move(rect.x + 20, rect.y + 2);

	GetFieldRect(2, rect);
	m_filestxt->SetSize(rect.x+2, rect.y+2, rect.width-4, rect.height-4);
	m_filesbmp->Move(rect.x + 2, rect.y + 2);
	m_filestxt->Move(rect.x + 20, rect.y + 2);
	
	GetFieldRect(3, rect);
	m_uptxt->SetSize(rect.x+2, rect.y+2, rect.width-20, rect.height-4);
	m_upbmp->Move(rect.x + 2, rect.y + 2);
	m_uptxt->Move(rect.x + 20, rect.y + 2);
	
	GetFieldRect(4, rect);
	m_downtxt->SetSize(rect.x+2, rect.y+2, rect.width-20, rect.height-4);
	m_downbmp->Move(rect.x + 2, rect.y + 2);
	m_downtxt->Move(rect.x + 20, rect.y + 2);
	
	GetFieldRect(5, rect);
	m_servertxt->SetSize(rect.x+2, rect.y+2, rect.width-20, rect.height-4);
	m_serverbmp->Move(rect.x + 2, rect.y + 2);
	m_servertxt->Move(rect.x + 20, rect.y + 2);
		
	event.Skip();
}

/**
 * Calculate the new widths for fields according to text+bitmap sizes,
 * plus some extra for it to look nice.
 */
void CStatusBar::UpdateFieldWidths() {
	int field_widths[6];

	field_widths[0] = -1;
	field_widths[1] = m_userbmp->GetSize().GetWidth()+m_usertxt->GetSize().GetWidth()+10;
	field_widths[2] = m_filesbmp->GetSize().GetWidth()+m_filestxt->GetSize().GetWidth()+10;
	field_widths[3] = m_upbmp->GetSize().GetWidth()+m_uptxt->GetSize().GetWidth()+10;
	field_widths[4] = m_downbmp->GetSize().GetWidth()+m_downtxt->GetSize().GetWidth()+10;
	field_widths[5] = m_serverbmp->GetSize().GetWidth()+m_servertxt->GetSize().GetWidth()+20;

	SetStatusWidths(6, field_widths);
}

/**
 * These methods set the values of various status bar fields.
 */
void CStatusBar::SetLogText(const wxString text) {
	m_logtxt->SetLabel(text);
}

void CStatusBar::SetUserCount(int users, int totalusers) {
	m_usertxt->SetLabel(wxString::Format(wxT("%d(%d)"), users, totalusers));
	UpdateFieldWidths();
}

void CStatusBar::SetFilesCount(int files, int totalfiles) {
	m_filestxt->SetLabel(wxString::Format(wxT("%d(%d)"), files, totalfiles));
	UpdateFieldWidths();
}

void CStatusBar::SetUpSpeed(int up) {
	m_uptxt->SetLabel(wxString::Format(wxT("%d kb/s"), up));
	UpdateFieldWidths();
}

void CStatusBar::SetDownSpeed(int down) {
	m_downtxt->SetLabel(wxString::Format(wxT("%d kb/s"), down));
	UpdateFieldWidths();
}

void CStatusBar::SetServerName(const wxString server) {
	m_servertxt->SetLabel(server);
	UpdateFieldWidths();
}
