/*
 *  This file is part of xMule2.
 *  Copyright (C) 2003 Alo Sarv <madcat_@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef __GNUG__
	#pragma implementation "SysTray.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#include "SysTray.h"

/* TaskbarIcon class is only supported under wxMSW */
#ifdef __WXMSW__

BEGIN_EVENT_TABLE(CSysTray,wxTaskBarIcon)
	EVT_TASKBAR_RIGHT_DOWN(CSysTray::ShowContextMenu)
	EVT_MENU(MENU_TRAY_HIDESHOW_GUI, CSysTray::HideShowGUI)
	EVT_MENU(MENU_TRAY_CHANGE_GUI_SETTINGS, CSysTray::ChangeGUISettings)
	EVT_MENU(MENU_TRAY_CHANGE_CORE_SETTINGS, CSysTray::ChangeCoreSettings)
	EVT_MENU(MENU_TRAY_HELP, CSysTray::ShowHelp)
	EVT_MENU(MENU_TRAY_QUIT, CSysTray::Quit)
END_EVENT_TABLE()

/**
 * Constructor for tray-icon. Sets the initial icon and text.
 */
CSysTray::CSysTray() : wxTaskBarIcon() {
	wxIcon icon;
	icon.CopyFromBitmap(img->mule);
	SetIcon(icon, wxT("xMule v2"));
}

/**
 * Destructor for our systray.
 */
CSysTray::~CSysTray() {
}

/**
 * In this method, we construct the context-menu (usually when the icon was
 * right-clicked, and show it. Note that wxTaskBarIcon::PopupMenu() only
 * takes one argument, different from wxWindow::PopupMenu(), which requires
 * position also.
 */
void CSysTray::ShowContextMenu(wxTaskBarIconEvent &event) {
	wxMenu *menu = new wxMenu();
	menu->Append(MENU_TRAY_HIDESHOW_GUI, _("Hide/Show Interface"));
	menu->Append(MENU_TRAY_CHANGE_GUI_SETTINGS, _("Change GUI Settings"));
	menu->Append(MENU_TRAY_CHANGE_CORE_SETTINGS, _("Change Core Settings"));
	menu->Append(MENU_TRAY_HELP, _("Help"));
	menu->Append(MENU_TRAY_QUIT, _("Quit"));
	PopupMenu(menu);
}

/**
 * This method will hide or show (iconize) the main dialog.
 */
void CSysTray::HideShowGUI(wxCommandEvent &event) {
	bool iconized = mainframe->IsIconized();
	mainframe->Iconize(!iconized);
}

/**
 * This method displays GUI Settings dialog.
 */
void CSysTray::ChangeGUISettings(wxCommandEvent &event) {
	CGUISettingsDlg *guisettings = new CGUISettingsDlg(
		NULL, -1, _("User Interface Settings"), 
		wxDefaultPosition, wxDefaultSize
	);
	guisettings->ShowModal();
	delete guisettings;
}

/**
 * This method displays Core Settings dialog.
 */
void CSysTray::ChangeCoreSettings(wxCommandEvent &event) {
	CCoreSettingsDlg *coredlg = new CCoreSettingsDlg(
		NULL, -1, _("Core Settings"),wxDefaultPosition, wxDefaultSize
	);
	coredlg->ShowModal();
	delete coredlg;
}

/**
 * This method is supposed to show help window (not implemented yet)
 */
void CSysTray::ShowHelp(wxCommandEvent &event) {
}

/**
 * This method will force the application to exit.
 */
void CSysTray::Quit(wxCommandEvent &event) {
	/**
	 * Altough wxWindows documentation recommends using Destroy() for deleting
	 * frames, we use delete operator here, simply because Destroy() can take
	 * up to several seconds to take effect, and it might look the quit command
	 * didn't have any effect
	 */
	delete mainframe;
}

#endif
