/*
 *  This file is part of xMule2.
 *  Copyright (C) 2003 Alo Sarv <madcat_@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef __GNUG__
	#pragma implementation "xMuleDlg.h"
#endif

// For compilers that support precompilation
#include "wx/wxprec.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#include "xMuleDlg.h"

/* Global classes */
class CServerWnd *serverwnd;
class CTransferWnd *transferwnd;
class CSearchWnd *searchwnd;
class CSharedFilesWnd *sharedfileswnd;
class CMessagesWnd *messageswnd;
class CStatisticsWnd *statisticswnd;
#ifdef __WXMSW__
	class CSysTray *systray;
#endif
class CStatusBar *statusbar;
/*
 * xMuleDlg class. The main dialog functions like Toolbar and Statusbar
 * stuff go in here.
 */

BEGIN_EVENT_TABLE(xMuleDlg,wxFrame)
	/* Shutdown events */
	EVT_CLOSE(xMuleDlg::OnCloseWindow)
	
	/* Main toolbar button events */
	EVT_TOOL(ID_BTN_CONNECT, xMuleDlg::ConnectToAnyServer)
	EVT_TOOL(ID_BTN_SERVERS, xMuleDlg::ShowServerWnd)
	EVT_TOOL(ID_BTN_TRANSFER, xMuleDlg::ShowTransferWnd)
	EVT_TOOL(ID_BTN_SEARCH, xMuleDlg::ShowSearchWnd)
	EVT_TOOL(ID_BTN_SHARED_FILES, xMuleDlg::ShowSharedFilesWnd)
	EVT_TOOL(ID_BTN_MESSAGES, xMuleDlg::ShowMessagesWnd)
	EVT_TOOL(ID_BTN_STATISTICS, xMuleDlg::ShowStatisticsWnd)
	EVT_TOOL(ID_BTN_GUISETTINGS, xMuleDlg::ShowGUISettingsDlg)

	/* Menubar events */
	EVT_MENU(ID_FILE_CONNECT, xMuleDlg::ConnectToAnyServer)
	EVT_MENU(ID_FILE_EXIT, xMuleDlg::OnCloseWindow)
	EVT_MENU(ID_EDIT_GUI_SETTINGS, xMuleDlg::ShowGUISettingsDlg)
	EVT_MENU(ID_VIEW_SERVERS, xMuleDlg::ShowServerWnd)
	EVT_MENU(ID_VIEW_TRANSFER, xMuleDlg::ShowTransferWnd)
	EVT_MENU(ID_VIEW_SEARCH, xMuleDlg::ShowSearchWnd)
	EVT_MENU(ID_VIEW_SHARED_FILES, xMuleDlg::ShowSharedFilesWnd)
	EVT_MENU(ID_VIEW_MESSAGES, xMuleDlg::ShowMessagesWnd)
	EVT_MENU(ID_VIEW_STATISTICS, xMuleDlg::ShowStatisticsWnd)
END_EVENT_TABLE()

/*
 * Main dialog constructor. Creates toolbar on the top, statusbar on the
 * bottom and loads the sub-dialogs (serverwnd, searchwnd etc).
 */
xMuleDlg::xMuleDlg(
	wxWindow *parent, wxWindowID id, const wxString &title,
	const wxPoint &position, const wxSize& size, long style,
	wxLocale &l
) 
: wxFrame( parent, id, title, position, size, style ), m_locale(l) {
	int height, width, posx, posy;
	start_up = true;

	/* Create toolbar */
	show_tool = true;
	m_config->Read(wxT("/General/Show toolbar"), &show_tool, true);
	if (show_tool) {
		CreateMyToolBar();
	}

	/* Create status bar */
	statusbar = new CStatusBar(this);
	SetStatusBar(statusbar);
	
	/* Create menu bar */
	bool show_menu;
	m_config->Read(wxT("/General/Show menubar"), &show_menu);
	if (show_menu) {
		CreateMenuBar();
	}

	/* Set application icon */
	wxIcon icon;
	icon.CopyFromBitmap(img->mule);
	SetIcon(icon);

	#ifdef __WXMSW__
		systray = new CSysTray();
	#endif

	/* This adds a new sizer in the frame to make switching pages easier. */
	m_mainsizer = ContentSizer(this, true);

	/**
	 * Read position and size settings, and apply them before 
	 * loading dialog pages. 
	 */
	m_config->Read(wxT("Frame/Height"), &height, 600);
	m_config->Read(wxT("Frame/Width"), &width, 800);
	SetSize(width, height);

	/**
	 * If X and Y positions were not found in config object, we will not
	 * use default values, but instead center the dialog (20,20 values 
	 * caused problems on OS X, better to center the whole thing and not 
	 * worry about it any more.
	 */
	bool isreadx = m_config->Read(wxT("Frame/PosX"), &posx, 20);
	bool isready = m_config->Read(wxT("Frame/PosY"), &posy, 20);
	if (!isreadx && !isready) {
		Center();
	} else {
		Move(posx, posy);
	}

	/* Everything else should be done now, finally load the pages itself. */
	LoadAndShowDialogPages();

}

/*
 * Destructor for xMuleDlg.
 */
xMuleDlg::~xMuleDlg() {
	/**
	 * Don't save the settings if we are iconized, otherwise we save our size
	 * and position to nulls or smth, and it seriously screws us on next 
	 * startup.
	 */
	if (!IsIconized()) {
		m_config->SetPath(wxT("/Frame"));
		m_config->Write(wxT("Height"), GetSize().GetHeight());
		m_config->Write(wxT("Width"), GetSize().GetWidth());
		m_config->Write(wxT("PosX"), GetPosition().x);
		m_config->Write(wxT("PosY"), GetPosition().y);
	}

	/* 
	 * If we are instructed to remember the last open page between sessions,
	 * write current active page to config file. This is done by determining
	 * which toolbar button is currently selected (stored in m_lastbtn variable)
	 * and substracting 1001 from that value to get 0 for serverwnd, 1 for 
	 * transferwnd etc, thus matching the codes we use from CGUISettingsDlg
	 * class.
	 */
	int remember_last;
	m_config->Read(wxT("/General/Remember last"), &remember_last, true);
	if (remember_last) {
		m_config->Write(wxT("/General/Start page"), m_lastbtn-1001);
	}

	#ifdef __WXMSW__
		systray->RemoveIcon();
		delete systray;
	#endif
	
	/* Clean up our mess before leaving */
	delete serverwnd;
	delete transferwnd;
	delete searchwnd;
	delete sharedfileswnd;
	delete messageswnd;
	delete statisticswnd;
}

/*
 * Changes content of the main frame by removing the old panel from m_mainsizer
 * and attaching new one. [start_up] defaults to false, and is required true
 * if we are calling this function on application start-up (don't hide old
 * (nonexistent) frame then).
 */
void xMuleDlg::SetActiveDialog(wxWindow* dlg) {
	/* On startup, we don't have activewnd here yet, so don't hide anything. */
	if (!start_up) {
		activewnd->Show(FALSE);
		m_mainsizer->Remove(activewnd);
	}
	m_mainsizer->Add(dlg,1,wxALIGN_LEFT|wxEXPAND|wxALL);
	dlg->Show(TRUE);
	activewnd=dlg;
	Layout();
	start_up = false;
}

/*
 * Creates the toolbar on the top end of the main dialog.
 */
void xMuleDlg::CreateMyToolBar() {
	int tool_align;
	m_config->Read(wxT("/General/Toolbar alignment"), &tool_align, wxTB_HORIZONTAL);
	/* Construct the toolbar object */
	tb = CreateToolBar(
 		tool_align|wxNO_BORDER|wxTB_TEXT|wxTB_FLAT|wxTB_3DBUTTONS
   	);
	tb->SetToolBitmapSize(wxSize(32, 32));
	tb->SetMargins( 2, 2 );

	/* Add the buttons */
	tb->AddTool(
		ID_BTN_CONNECT, _("Connect"), img->btn_connect, 
		wxNullBitmap, wxITEM_NORMAL, _("Connect to any server") 
	);
	tb->AddSeparator();
	tb->AddTool(
		ID_BTN_SERVERS, _("Servers"), img->btn_servers, 
		wxNullBitmap, wxITEM_CHECK, _("Server list")
	);
	tb->AddTool( 
		ID_BTN_SEARCH, _("Search"), img->btn_search, 
		wxNullBitmap, wxITEM_CHECK, _("Search files") 
	);
	tb->AddTool(
		ID_BTN_TRANSFER, _("Transfer"), img->btn_transfer, 
		wxNullBitmap, wxITEM_CHECK, _("File Transfer")
	);
	tb->AddTool(
		ID_BTN_SHARED_FILES, _("Shared Files"), img->btn_shared, 
		wxNullBitmap, wxITEM_CHECK, _("Show shared files")
	);
	tb->AddTool(
		ID_BTN_MESSAGES, _("Messages"), img->btn_messages, 
		wxNullBitmap, wxITEM_CHECK, _("Chatting") 
	);
	tb->AddTool(
		ID_BTN_STATISTICS, _("Statistics"), img->btn_statistics, 
		wxNullBitmap, wxITEM_CHECK, _("Show live statistics") );
	tb->AddSeparator();
	tb->AddTool(
		ID_BTN_GUISETTINGS, _("Preferences"), img->btn_guisettings, 
		wxNullBitmap, wxITEM_NORMAL, _("Modify settings")
	);

	/* And go live */
	tb->Realize();
}

/*
 * CloseWindow event function. This should destroy all open dialogs.
 */
void xMuleDlg::OnCloseWindow( wxCloseEvent &event ){
	/* Read from config if we should ask for permission before closing. */
	bool prompt_exit;
	m_config->Read(wxT("/General/Prompt on exit"), &prompt_exit, true); 

	if (prompt_exit) {
		if (wxMessageBox(_("Close xmule2-ui-wx?"), _("Question"), 
			wxYES_NO|wxCENTRE|wxICON_QUESTION) == wxNO) 
		{
			return;
		}
	}
	/* We reach this line if prompt_exit was false, 
	 * or Yes was clicked in messagebox 
	 */
	Destroy();
}

/**
 * Shows server page
 */
void xMuleDlg::ShowServerWnd(wxCommandEvent &event) {
	if (show_tool) {
		tb->ToggleTool(m_lastbtn, false);
		tb->ToggleTool(ID_BTN_SERVERS, true);
	}
	m_lastbtn = ID_BTN_SERVERS;
	SetActiveDialog(serverwnd);
	/**
	 * We send the server page a sizeevent to get the log boxes columns get
	 * correct column widths (they are calculated dynamically according to
	 * listctrl width.
	 */
	wxSizeEvent null_event;
	serverwnd->OnSize(null_event);
}

/**
 * Shows transfer page
 */
void xMuleDlg::ShowTransferWnd(wxCommandEvent &event) {
	if (show_tool) {
		tb->ToggleTool(m_lastbtn, false);
		tb->ToggleTool(ID_BTN_TRANSFER, true);
	}
	m_lastbtn = ID_BTN_TRANSFER;
	SetActiveDialog(transferwnd);
}

/**
 * Shows search page
 */
void xMuleDlg::ShowSearchWnd(wxCommandEvent &event) {
	if (show_tool) {
		tb->ToggleTool(m_lastbtn, false);
		tb->ToggleTool(ID_BTN_SEARCH, true);
	}
	m_lastbtn = ID_BTN_SEARCH;
	SetActiveDialog(searchwnd);
}

/**
 * Shows shared files page
 */
void xMuleDlg::ShowSharedFilesWnd(wxCommandEvent &event) {
	if (show_tool) {
		tb->ToggleTool(m_lastbtn, false);
		tb->ToggleTool(ID_BTN_SHARED_FILES, true);
	}
	m_lastbtn = ID_BTN_SHARED_FILES;
	SetActiveDialog(sharedfileswnd);
}

/**
 * Shows messages page
 */
void xMuleDlg::ShowMessagesWnd(wxCommandEvent &event) {
	if (show_tool) {
		tb->ToggleTool(m_lastbtn, false);
		tb->ToggleTool(ID_BTN_MESSAGES, true);
	}
	m_lastbtn = ID_BTN_MESSAGES;
	SetActiveDialog(messageswnd);
}

/**
 * Shows statistics page
 */
void xMuleDlg::ShowStatisticsWnd(wxCommandEvent &event) {
	if (show_tool) {
		tb->ToggleTool(m_lastbtn, false);
		tb->ToggleTool(ID_BTN_STATISTICS, true);
	}
	m_lastbtn = ID_BTN_STATISTICS;
	SetActiveDialog(statisticswnd);
}

/**
 * Sends command to core to connect to any server
 */
void xMuleDlg::ConnectToAnyServer(wxCommandEvent &event) {
}

/**
 * Displays GUI settings dialog
 */
void xMuleDlg::ShowGUISettingsDlg(wxCommandEvent &event) {
	CGUISettingsDlg *guisettings = new CGUISettingsDlg(
		NULL, -1, _("Preferences"), 
		wxDefaultPosition, wxDefaultSize
	);
	guisettings->ShowModal();
	delete guisettings;
}

/**
 * Loads all dialog pages, and displays either the one set from preferences,
 * last one, or default (server) page.
 */
void xMuleDlg::LoadAndShowDialogPages() {
	int start_page;

	serverwnd = new CServerWnd(this);
	searchwnd = new CSearchWnd(this);
	transferwnd = new CTransferWnd(this);
	sharedfileswnd = new CSharedFilesWnd(this);
	messageswnd = new CMessagesWnd(this);
	statisticswnd = new CStatisticsWnd(this);

	m_config->Read(wxT("/General/Start page"), &start_page, 0);
	wxCommandEvent null_event;
	switch (start_page) {
		case 0:	ShowServerWnd(null_event); break;
		case 1:	ShowTransferWnd(null_event); break;
		case 2: ShowSearchWnd(null_event); break;
		case 3: ShowSharedFilesWnd(null_event); break;
		case 4: ShowMessagesWnd(null_event); break;
		case 5: ShowStatisticsWnd(null_event); break;
		default: ShowServerWnd(null_event); break;
	}
}

/**
 * Generates our menubar.
 */
void xMuleDlg::CreateMenuBar() {

	wxMenu *file_menu = new wxMenu();
	file_menu->Append(ID_FILE_CONNECT, _("Connect"));
	file_menu->Append(ID_FILE_EXIT, _("Quit"));
	
	wxMenu *edit_menu = new wxMenu();
	edit_menu->Append(ID_EDIT_GUI_SETTINGS, _("Preferences"));

	wxMenu *view_menu = new wxMenu();
	view_menu->Append(ID_VIEW_SERVERS, _("Servers"));
	view_menu->Append(ID_VIEW_TRANSFER, _("Transfer"));
	view_menu->Append(ID_VIEW_SEARCH, _("Search"));
	view_menu->Append(ID_VIEW_SHARED_FILES, _("Shared Files"));
	view_menu->Append(ID_VIEW_MESSAGES, _("Messages"));
	view_menu->Append(ID_VIEW_STATISTICS, _("Statistics"));

	wxMenu *help_menu = new wxMenu();
	help_menu->Append(ID_HELP_HELP, _("Help"));
	help_menu->Append(ID_HELP_ABOUT, _("About..."));

	wxMenuBar *menubar = new wxMenuBar();
	menubar->Append(file_menu, _("File"));
	menubar->Append(edit_menu, _("Edit"));
	menubar->Append(view_menu, _("View"));
	menubar->Append(help_menu, _("Help"));

	SetMenuBar(menubar);
}
