/*
 *  This file is part of xMule2.
 *  Copyright (C) 2003 Alo Sarv <madcat_@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef __GNUG__
	#pragma implementation "xMuleGUI.h"
#endif

// For compilers that support precompilation
#include "wx/wxprec.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#include "xMuleGUI.h"

class xMuleDlg *mainframe;

/* 
 * Base class for xMule wx user interface. Everything else starts from here.
 */

IMPLEMENT_APP(xMuleGUI)

class CImages *img;
wxConfigBase *m_config;

/*
 * Main program initialization. Command arguments should be checked here, and
 * main xMule Dialog drawn.
 */
bool xMuleGUI::OnInit() {
	int show_splash;
	
	SetVendorName(wxT("Alo Sarv"));
	SetAppName(wxT("xmule2-ui-wx"));
	
	::wxInitAllImageHandlers();

	img = new CImages();

	m_config = wxConfigBase::Get();
	m_config->Read(wxT("/General/Show splashscreen"), &show_splash);

	if (show_splash) {
		/* Display the splash screen */
		splash = new wxSplashScreen(
			img->logo, wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT, 
			2000, NULL, -1
		);
	}

	Localize();
	
	/* Display the main dialog */
	mainframe = new xMuleDlg(
		NULL, -1, APPVER_LONG, wxDefaultPosition,
		wxDefaultSize, wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE,
		m_locale
	);
	mainframe->Show( TRUE );

	if (show_splash) {
		splash->Raise();
	}

	return TRUE;
}

/*
 * Main program shutdown function. If anything needs to be cleaned up, do it 
 * here.
 */
int xMuleGUI::OnExit() {
	/* Clean up our mess before leaving */
	delete m_config->Set(NULL);
	return 0;
}

/**
 * First we call CheckAndAddSupportedLangs() to add built-in languages to config
 * object. Then we add catalog prefixes (note that we need ::Resources:lang also
 * to be compatible with Mac OS packaging standards). Then simply read active
 * language from config object, and add the catalog.
 */
void xMuleGUI::Localize() {
	wxString curlang;

	CheckAndAddSupportedLangs();

	/* Set localization files directories */
	m_locale.AddCatalogLookupPathPrefix(wxT("lang/"));
	#if defined(__WXMAC__) && !defined(__MACH__)
		m_locale.AddCatalogLookupPathPrefix(wxT("::Resources:lang"));
	#endif

	m_config->SetPath(wxT("/General/Lang"));
	m_config->Read(wxT("CurLang"), &curlang);

	m_locale.Init(GetLangType(curlang));
	m_locale.AddCatalog(curlang);
}

/**
 * This method's purpose is ensuring that all supported languages 
 * (translations shipped with releases) are added to config file. This is done
 * by creating a list of supported languages (supportedlangs), then reading
 * all languages from config object (conflangs), and then comparing those two
 * arrays, adding missing elements to conflangs. Finally, the new languages
 * array (conflangs) is written back into config object.
 */
void xMuleGUI::CheckAndAddSupportedLangs() {
	wxArrayString conflangs, supportedlangs;
	wxString tmpname;
	int numlang;

	/* List of supported langs - add new ones to the bottom */
	supportedlangs.Add(wxT("Default"));
	supportedlangs.Add(wxT("German"));
	supportedlangs.Add(wxT("Estonian"));

	m_config->SetPath(wxT("/General/Lang"));

	m_config->Read(wxT("NumLang"), &numlang, 1);

	/* Loop through config and read all languages to conflangs array */
	for (int i=0; i<numlang; i++) {
		m_config->Read(
			wxString::Format(wxT("%d"), i), 
			&tmpname, 
			wxT("Default")
		);
		conflangs.Add(tmpname);
	}

	/* Loop through supportedlangs array and add missing items to conflangs */
	for (uint i=0; i<supportedlangs.GetCount(); i++) {
		if (conflangs.Index(supportedlangs.Item(i)) == wxNOT_FOUND) {
			conflangs.Add(supportedlangs.Item(i));
		}
	}

	/* Write conflangs array back to config */
	for (uint i=0; i<conflangs.GetCount(); i++) {
		m_config->Write(
			wxString::Format(wxT("%d"), i),
			conflangs.Item(i)
		);
	}
	m_config->Write(wxT("NumLang"), (int)conflangs.GetCount());

}

/**
 * This method's purpose is to determine the wxWindows constant for given 
 * language (affects dates/times/currencies etc). Input variable [lang]
 * should be any language name. On success, correct localization setting
 * is returned, while if no match is found, wxLANGUAGE_DEFAULT is
 * returned (use system setting).
 */
int xMuleGUI::GetLangType(const wxString lang) {
	if (lang == "German") {
		return wxLANGUAGE_GERMAN;
	} else if (lang == "Estonian") {
		return wxLANGUAGE_ESTONIAN;
	} else {
		return wxLANGUAGE_DEFAULT;
	}
}
